// CViewport.cpp
// by Comanche

#include "CViewport.h"
#include "CMain.h"
#include "CGraphics.h"
#include "CButton.h"
#include "CLog.h"
#include "CVoxel.h"
#include "CColor.h"

extern CMain GameMain;
extern CGraphics Graphics;
extern CLog Log;
extern CVoxel Voxel;
extern CColor Color;

//////////////////////////////////////////////////////////////////////////////////

CViewport::CViewport(int x,int y,int M)
{
	Vx=x; Vy=y;
	Vx2=Vx+195; Vy2=Vy+195;
	Mode=M;
	Voxel.FrontPos=48;

	if(Mode!=VIEWPORT3D)
	{
		ViewportButtons[0] = new CButton(x+200,y,x+210,y+98,"","");
		ViewportButtons[1] = new CButton(x+200,y+99,x+210,y+196,"","");
	}
}

//////////////////////////////////////////////////////////////////////////////////

CViewport::~CViewport()
{
	if(Mode!=VIEWPORT3D)
	{
		delete ViewportButtons[0];
		delete ViewportButtons[1];
	}
}

//////////////////////////////////////////////////////////////////////////////////

void CViewport::UseViewport()
{
	if(Mode!=VIEWPORT3D)
	{
		if(ViewportButtons[0]->TestIfPressed())
		{
			if(Mode==VIEWPORTTOP)			
				if(Voxel.TopPos>0) 
				{
					Voxel.TopPos--;
					Graphics.Lock();
					for(int i=0;i<49;i++)
						for(int i2=0;i2<49;i2++)
						{
						Graphics.FullQuader(Vx+(i*4)+1,Vy+(i2*4)+1,
											Vx+(i*4)+3,Vy+(i2*4)+3,
											Voxel.Voxels[i][i2][Voxel.TopPos].rgbtRed,
											Voxel.Voxels[i][i2][Voxel.TopPos].rgbtGreen,
											Voxel.Voxels[i][i2][Voxel.TopPos].rgbtBlue);											
						}
					Graphics.UnLock();
				}
			if(Mode==VIEWPORTFRONT)
				if(Voxel.FrontPos>0) 
				{
					Voxel.FrontPos--;				
					Graphics.Lock();
					for(int i=0;i<49;i++)
						for(int i2=0;i2<49;i2++)
						{
						Graphics.FullQuader(Vx+(i*4)+1,Vy+(i2*4)+1,
											Vx+(i*4)+3,Vy+(i2*4)+3,
											Voxel.Voxels[Voxel.FrontPos][i][i2].rgbtRed,
											Voxel.Voxels[Voxel.FrontPos][i][i2].rgbtGreen,
											Voxel.Voxels[Voxel.FrontPos][i][i2].rgbtBlue);											
						}
					Graphics.UnLock();
				}
			if(Mode==VIEWPORTLEFT)
			{				
				if(Voxel.LeftPos>0)
				{
					Voxel.LeftPos--;
					Graphics.Lock();
					for(int i=0;i<49;i++)
						for(int i2=0;i2<49;i2++)
						{
						Graphics.FullQuader(Vx+(i*4)+1,Vy+(i2*4)+1,
											Vx+(i*4)+3,Vy+(i2*4)+3,
											Voxel.Voxels[i][Voxel.LeftPos][i2].rgbtRed,
											Voxel.Voxels[i][Voxel.LeftPos][i2].rgbtGreen,
											Voxel.Voxels[i][Voxel.LeftPos][i2].rgbtBlue);											
						}
					Graphics.UnLock();
				}
			}
		}

		//////////////////////////////////////////////////////////////////////

		if(ViewportButtons[1]->TestIfPressed())
		{
			if(Mode==VIEWPORTTOP)			
				if(Voxel.TopPos<48)
				{
					Voxel.TopPos++;								
					Graphics.Lock();
					for(int i=0;i<49;i++)
						for(int i2=0;i2<49;i2++)
						{
						Graphics.FullQuader(Vx+(i*4)+1,Vy+(i2*4)+1,
											Vx+(i*4)+3,Vy+(i2*4)+3,
											Voxel.Voxels[i][i2][Voxel.TopPos].rgbtRed,
											Voxel.Voxels[i][i2][Voxel.TopPos].rgbtGreen,
											Voxel.Voxels[i][i2][Voxel.TopPos].rgbtBlue);											
						}
					Graphics.UnLock();
				}
			if(Mode==VIEWPORTFRONT)
				if(Voxel.FrontPos<48) 
				{
					Voxel.FrontPos++;				
					Graphics.Lock();
					for(int i=0;i<49;i++)
						for(int i2=0;i2<49;i2++)
						{
						Graphics.FullQuader(Vx+(i*4)+1,Vy+(i2*4)+1,
											Vx+(i*4)+3,Vy+(i2*4)+3,
											Voxel.Voxels[Voxel.FrontPos][i][i2].rgbtRed,
											Voxel.Voxels[Voxel.FrontPos][i][i2].rgbtGreen,
											Voxel.Voxels[Voxel.FrontPos][i][i2].rgbtBlue);											
						}
					Graphics.UnLock();
				}
			if(Mode==VIEWPORTLEFT)
				if(Voxel.LeftPos<48) 
				{
					Voxel.LeftPos++;				
					Graphics.Lock();
					for(int i=0;i<49;i++)
						for(int i2=0;i2<49;i2++)
						{
						Graphics.FullQuader(Vx+(i*4)+1,Vy+(i2*4)+1,
											Vx+(i*4)+3,Vy+(i2*4)+3,
											Voxel.Voxels[i][Voxel.LeftPos][i2].rgbtRed,
											Voxel.Voxels[i][Voxel.LeftPos][i2].rgbtGreen,
											Voxel.Voxels[i][Voxel.LeftPos][i2].rgbtBlue);											
						}
					Graphics.UnLock();
				}
		}
	}

	if(Mode==VIEWPORTLEFT)
	{
		Graphics.Lock();
		Graphics.FullQuader(Vx+(Voxel.FrontPos*4)-4,Vy+199,
							Vx+(Voxel.FrontPos*4)+7,Vy+203,150,150,150);
		Graphics.FullQuader(Vx+(Voxel.FrontPos*4)+1,Vy+199,
							Vx+(Voxel.FrontPos*4)+2,Vy+203,100,100,100);
		Graphics.VLine(Vx+(Voxel.FrontPos*4),Vy+200,3,100,100,100);

		Graphics.FullQuader(Vx-7,Vy+(Voxel.TopPos*4)-4,
							Vx-3,Vy+(Voxel.TopPos*4)+7,150,150,150);
		Graphics.FullQuader(Vx-7,Vy+(Voxel.TopPos*4)+1,
							Vx-3,Vy+(Voxel.TopPos*4)+2,100,100,100);
		Graphics.HLine(Vx-4,Vy+(Voxel.TopPos*4),3,100,100,100);
		Graphics.UnLock();
		
		if(GameMain.CheckMouseRange(Vx,Vy,Vx2,Vy2,FALSE))
		{		
			if((GameMain.MouseX-Vx)%4!=0
				&& (GameMain.MouseY-Vy)%4!=0)
			{
				Voxel.ViewX=(GameMain.MouseX-Vx)/4;
				Voxel.ViewY=(GameMain.MouseY-Vy)/4;

				if(GameMain.MouseB[0])
				{
					Voxel.Voxels[Voxel.ViewX][Voxel.LeftPos][Voxel.ViewY] 
						= Color.color;
					Graphics.Lock();
					Graphics.FullQuader(Vx+(Voxel.ViewX*4)+1,Vy+(Voxel.ViewY*4)+1,
										Vx+(Voxel.ViewX*4)+3,Vy+(Voxel.ViewY*4)+3,
										Color.color.rgbtRed,Color.color.rgbtGreen,
										Color.color.rgbtBlue);
					Graphics.UnLock();
				}
			}
		}		

		return;
	}

	if(Mode==VIEWPORTTOP)
	{
		Graphics.Lock();
		
		Graphics.FullQuader(Vx+(Voxel.LeftPos*4)-4,Vy+199,
							Vx+(Voxel.LeftPos*4)+7,Vy+203,150,150,150);
		Graphics.FullQuader(Vx+(Voxel.LeftPos*4)+1,Vy+199,
							Vx+(Voxel.LeftPos*4)+2,Vy+203,100,100,100);
		Graphics.VLine(Vx+(Voxel.LeftPos*4),Vy+200,3,100,100,100);
		
		Graphics.FullQuader(Vx-7,Vy+(Voxel.FrontPos*4)-4,
							Vx-3,Vy+(Voxel.FrontPos*4)+7,150,150,150);
		Graphics.FullQuader(Vx-7,Vy+(Voxel.FrontPos*4)+1,
							Vx-3,Vy+(Voxel.FrontPos*4)+2,100,100,100);
		Graphics.HLine(Vx-4,Vy+(Voxel.FrontPos*4),3,100,100,100);
		Graphics.UnLock();

		if(GameMain.CheckMouseRange(Vx,Vy,Vx2,Vy2,FALSE))
		{
			if((GameMain.MouseX-Vx)%4!=0
				&& (GameMain.MouseY-Vy)%4!=0)
			{
				Voxel.ViewX=(GameMain.MouseX-Vx)/4;
				Voxel.ViewY=(GameMain.MouseY-Vy)/4;
				
				if(GameMain.MouseB[0])
				{
					Voxel.Voxels[Voxel.ViewY][Voxel.ViewX][Voxel.TopPos] 
						= Color.color;
					Graphics.Lock();
					Graphics.FullQuader(Vx+(Voxel.ViewX*4)+1,Vy+(Voxel.ViewY*4)+1,
										Vx+(Voxel.ViewX*4)+3,Vy+(Voxel.ViewY*4)+3,
										Color.color.rgbtRed,Color.color.rgbtGreen,
										Color.color.rgbtBlue);
					Graphics.UnLock();
				}
			}
		}

		return;
	}

	if(Mode==VIEWPORTFRONT)
	{
		Graphics.Lock();
		Graphics.FullQuader(Vx+(Voxel.LeftPos*4)-4,Vy+199,
							Vx+(Voxel.LeftPos*4)+7,Vy+203,150,150,150);
		Graphics.FullQuader(Vx+(Voxel.LeftPos*4)+1,Vy+199,
							Vx+(Voxel.LeftPos*4)+2,Vy+203,100,100,100);
		Graphics.VLine(Vx+(Voxel.LeftPos*4),Vy+200,3,100,100,100);

		Graphics.FullQuader(Vx-7,Vy+(Voxel.TopPos*4)-4,
							Vx-3,Vy+(Voxel.TopPos*4)+7,150,150,150);
		Graphics.FullQuader(Vx-7,Vy+(Voxel.TopPos*4)+1,
							Vx-3,Vy+(Voxel.TopPos*4)+2,100,100,100);
		Graphics.HLine(Vx-4,Vy+(Voxel.TopPos*4),3,100,100,100);
		Graphics.UnLock();

		if(GameMain.CheckMouseRange(Vx,Vy,Vx2,Vy2,FALSE))
		{
			if((GameMain.MouseX-Vx)%4!=0
				&& (GameMain.MouseY-Vy)%4!=0)
			{
				Voxel.ViewX=(GameMain.MouseX-Vx)/4;
				Voxel.ViewY=(GameMain.MouseY-Vy)/4;
			}
		}

		return;
	}

	if(Mode==VIEWPORT3D)
	{
		return;
	}	
}

//////////////////////////////////////////////////////////////////////////////////

void CViewport::RedrawViewport()
{
	Graphics.BlackQuader(Vx+1,Vy+1,Vx+196,Vy+196);
	Graphics.Lock();
	Graphics.Quader(Vx,Vy,Vx+196,Vy+196,120,120,120);
	Graphics.UnLock();
	Graphics.OSGetDC();
	if (Mode==VIEWPORT3D) Graphics.Text(Vx+98,Vy-16,"3D");
	if (Mode==VIEWPORTFRONT) Graphics.Text(Vx+98,Vy-16,"Front");
	if (Mode==VIEWPORTLEFT) Graphics.Text(Vx+98,Vy-16,"Left");
	if (Mode==VIEWPORTTOP) Graphics.Text(Vx+98,Vy-16,"Top");
	Graphics.OSReleaseDC();

	if(Mode!=VIEWPORT3D)
	{
		ViewportButtons[0]->RedrawButton();
		ViewportButtons[1]->RedrawButton();
	}
}
