// PEVoxEd.cpp
// by Comanche

#include "defines.h"
#include "CMain.h"
#include "CLog.h"
#include "CGraphics.h"
#include "CButton.h"
#include "CLandScape.h"
#include "CMenu.h"
#include "CTime.h"
#include "CTexture.h"
#include "CViewport.h"
#include "CColor.h"
#include "CVoxel.h"
#include "CLiquid.h"
#include "CCheckBox.h"

//////////////////////////////////////////////////////////////////////////////////

CMain GameMain;
CLog Log;
CGraphics Graphics;
CLandScape Land;
CMenu Menu;
CTime Time;
CColor Color;
CButton *Buttons[60];
CCheckBox *CheckBoxes[20];
CViewport *Viewports[4];
CTexture *Textures[1024];
CLiquid *Liquids[128];
CTexture *Masks[3];
CVoxel Voxel;

//////////////////////////////////////////////////////////////////////////////////

HWND		main_window_handle = NULL;	// save the window handle
HINSTANCE	main_instance = NULL;		// save the instance
BOOL		bActive=TRUE,				// is the aplication minimized?
			bMinimized=FALSE;
BYTE		*video_buffer;
int			WordsPerLine;
			

//////////////////////////////////////////////////////////////////////////////////


LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
PAINTSTRUCT	ps;		   // used in WM_PAINT
HDC			hdc;	   // handle to a device context

switch(msg)
	{	
	case WM_CREATE: 
        {
		return(0);
		} break;

    case WM_PAINT:
         {
         hdc = BeginPaint(hwnd,&ps);
         EndPaint(hwnd,&ps);
         return(0);
        } break;

	case WM_DESTROY: 
		{
		PostQuitMessage(0);
		return(0);
		} break;

    case WM_ACTIVATEAPP:
		{
        bActive = (BOOL)wparam;
		} break;

	default:break;

    } // end switch
return (DefWindowProc(hwnd, msg, wparam, lparam));
} // end WinProc

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{
WNDCLASS winclass;	// this will hold the class we create
HWND	 hwnd;		// generic window handle
MSG		 msg;		// generic message

Log.WriteStatusToLogFile("> Initialize Window");

winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(hinstance,MAKEINTRESOURCE(IDI_ICON));
winclass.hCursor		= LoadCursor(hinstance,IDC_ARROW);
winclass.lpszMenuName	= NULL; 
winclass.lpszClassName	= WINDOW_CLASS_NAME;

if (!RegisterClass(&winclass))
{
	Log.WriteErrorToLogFile("> Error: Register window class");
	return(0);
}
Log.WriteToLogFile("> Register window class: OK");

if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME, // class
						  "VoxEd 2.0",	 // title
						  WS_POPUP | WS_VISIBLE, 
						  0,0,	   // x,y
						  WINDOW_WIDTH,  // width
                          WINDOW_HEIGHT, // height
						  NULL,	   // handle to parent 
						  NULL,	   // handle to menu
						  hinstance,// instance
						  NULL)))	// creation parms
{
	Log.WriteErrorToLogFile("> Error: Create window");
	return(FALSE);
}
Log.WriteToLogFile("> Create window: OK");

main_window_handle = hwnd;
main_instance      = hinstance;

Log.WriteStatusToLogFile("> GameInit: Initialize MMX DirectDraw graphic subsystem");
if(!GameMain.GameInit(main_window_handle))		// Game initializing
{
	GameMain.GameShutdown();		// an error
	return(FALSE);
}
Log.WriteToLogFile("> GameInit: OK");

Log.WriteStatusToLogFile("> MouseInit: Initialize mouse");
if(!GameMain.MouseInit(hwnd))	// Mouse initializing
{
	GameMain.GameShutdown();		// an error
	return(FALSE);
}
Log.WriteToLogFile("> MouseInit: OK");

Log.WriteStatusToLogFile("> KeyboardInit: Intialize keyboard");
if(!GameMain.KeyboardInit(hwnd))	// Keyboard initializing
{
	GameMain.GameShutdown();
	return(FALSE);
}
Log.WriteToLogFile("> KeyboardInit: OK");

Log.WriteStatusToLogFile("> LoadTextures: Loading textures");
if(!GameMain.LoadTextures())
{
	GameMain.GameShutdown();
	return(FALSE);
}
Log.WriteToLogFile("> LoadTextures: OK");

Log.WriteStatusToLogFile("> GameStart: Draw window graphics");
if(!GameMain.GameStart())		// Window Graphics
{
	GameMain.GameShutdown();		// an error
	return(FALSE);
}
Log.WriteToLogFile("> GameStart: OK");

// enter main event loop

Log.WriteStatusToLogFile("> Entering message loop");

while( 1 )
{
	if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
    {	// Messages abarbeiten
		if( !GetMessage( &msg, NULL, 0, 0 ) )
        {
			GameMain.GameShutdown();
			return msg.wParam;
        }
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    else if ( bActive )	// Keine Messages und aktiv: Game_Main
    {
		if (bMinimized) 
		{
			if (!GameMain.GameRestart())
			{
				GameMain.GameShutdown();
				return msg.wParam;
			}
			Log.WriteToLogFile("> Game_Restart: OK");
			Log.WriteToLogFile("> Maximized");
			bMinimized=FALSE;
		}
		if (!GameMain.GameMain())
		{
			GameMain.GameShutdown();		// an error: shutdown the Game
			return(FALSE);
		} // end if
    }
    else	// Nichts zu tun: Warten
    {
		if (!bMinimized)
			Log.WriteToLogFile("> Minimized");
		WaitMessage();
		bMinimized=TRUE;
	}
}

// shutdown game and release all resources

} // end WinMain